// SPDX-FileCopyrightText: 2024 Yury Gubich <blue@macaw.me>
// SPDX-License-Identifier: GPL-3.0-or-later

#pragma once

#include <set>
#include <map>
#include <functional>
#include <mutex>
#include <atomic>

#include <sys/select.h>

#include "log/loggable.h"
#include "log/logger.h"

class Loop : Log::Loggable {
public:
    typedef std::function<void()> Callback;
    typedef std::map<int, Callback> Handlers;

    explicit Loop(const Log::Logger& logger);
    Loop(const Loop&) = delete;
    Loop& operator=(const Loop&) = delete;
    Loop(Loop&&) noexcept = delete;
    Loop& operator=(Loop&&) noexcept = delete;
    ~Loop();

    void run();
    void stop();
    void addDescriptor(int descriptor, Callback&& handler);

    void removeDescriptor(int descriptor);

private:
    void wake() const;
    void drain(fd_set* readfds) const;
    void runCallbacks(fd_set* readfds) const;
    int setFDsAndFindMax(fd_set* readfds) const;
    void syncHandlers();

private:
    int wakePipe[2];
    Handlers handlers;
    Handlers handlersToAdd;
    std::set<int> descriptorsToRemove;
    mutable std::mutex mutex;
    std::atomic<bool> running;

private:
    static void registerInstance(Loop* loop);
    static void unregisterInstance(Loop* loop);
    static void signalHandler(int signum);

    static inline std::set<Loop*> instances;
    static inline std::mutex instanceMutex;

};
