// SPDX-FileCopyrightText: 2024 Yury Gubich <blue@macaw.me>
// SPDX-License-Identifier: GPL-3.0-or-later

#include "log.h"

static constexpr std::string_view gray = "\033[90m";
static constexpr std::string_view blue = "\033[34m";
static constexpr std::string_view green = "\033[32m";
static constexpr std::string_view yellow = "\033[33m";
static constexpr std::string_view red = "\033[31m";
static constexpr std::string_view magenta = "\033[35m";

gloox::LogLevel Log::convertGloox(const Level level) {
    switch (level) {
        case Level::trace:
            return gloox::LogLevelDebug;
        case Level::debug:
            return gloox::LogLevelWarning;
        case Level::info:
            return gloox::LogLevelWarning;
        case Level::warning:
            return gloox::LogLevelWarning;
        case Level::error:
            return gloox::LogLevelError;
        case Level::fatal:
            return gloox::LogLevelError;
    }

    return gloox::LogLevelWarning;
}

Log::Level Log::convertGloox(const gloox::LogLevel level) {
    switch (level) {
        case gloox::LogLevelDebug:
            return Level::trace;
        case gloox::LogLevelError:
            return Level::error;
        case gloox::LogLevelWarning:
            return Level::warning;
    };

    return Level::warning;
}

boost::log::trivial::severity_level Log::convertBoost(const Level level) {
    switch (level) {
        case Level::trace:
            return boost::log::trivial::trace;
        case Level::debug:
            return boost::log::trivial::debug;
        case Level::info:
            return boost::log::trivial::info;
        case Level::warning:
            return boost::log::trivial::warning;
        case Level::error:
            return boost::log::trivial::error;
        case Level::fatal:
            return boost::log::trivial::fatal;
    }

    return boost::log::trivial::error;
}

Log::Level Log::convertBoost(const boost::log::trivial::severity_level level) {
    switch (level) {
        case boost::log::trivial::trace:
            return Level::trace;
        case boost::log::trivial::debug:
            return Level::debug;
        case boost::log::trivial::info:
            return Level::info;
        case boost::log::trivial::warning:
            return Level::warning;
        case boost::log::trivial::error:
            return Level::error;
        case boost::log::trivial::fatal:
            return Level::fatal;
    }

    return Level::error;
}

std::string_view Log::levelColor(const Level level) {
    switch (level) {
        case Level::trace:
            return gray;
        case Level::debug:
            return blue;
        case Level::info:
            return green;
        case Level::warning:
            return yellow;
        case Level::error:
            return red;
        case Level::fatal:
            return magenta;
    }

    return red;
}
